#!/usr/bin/python
# -*- coding: utf-8 -*-

from pycoproc_1 import Pycoproc
from MFRC630 import MFRC630
from LIS2HH12 import LIS2HH12
from LTR329ALS01 import LTR329ALS01
import time
import pycom


# This is the default key for an unencrypted MiFare card
CARDkey = [ 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF ]
# CARDkey = [ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ]
DECODE_CARD = True

py = Pycoproc(Pycoproc.PYSCAN)
nfc = MFRC630(py)
lt = LTR329ALS01(py)
li = LIS2HH12(py)

pybytes_enabled = False
if 'pybytes' in globals():
    if(pybytes.isconnected()):
        print('Pybytes is connected, sending signals to Pybytes')
        pybytes_enabled = True

RGB_BRIGHTNESS = 0x8

RGB_RED = (RGB_BRIGHTNESS << 16)
RGB_GREEN = (RGB_BRIGHTNESS << 8)
RGB_BLUE = (RGB_BRIGHTNESS)

counter = 0

def send_sensor_data(name, timeout):
    if(pybytes_enabled):
        while(True):
            pybytes.send_signal(2, lt.light())
            pybytes.send_signal(3, li.acceleration())
            time.sleep(timeout)

# Make sure heartbeat is disabled before setting RGB LED
pycom.heartbeat(False)

# Initialise the MFRC630 with some settings
nfc.mfrc630_cmd_init()

print('Scanning for cards')
while True:
    # Send REQA for ISO14443A card type
    atqa = nfc.mfrc630_iso14443a_WUPA_REQA(nfc.MFRC630_ISO14443_CMD_REQA)
    if (atqa != 0):
        # A card has been detected, read UID
        print('A card has been detected, reading its UID ...')
        uid = bytearray(10)
        uid_len = nfc.mfrc630_iso14443a_select(uid)
        print('UID has length {}'.format(uid_len))
        if (uid_len > 0):
            # A valid UID has been detected, print details
            counter += 1
            print("%d\tUID [%d]: %s" % (counter, uid_len, nfc.format_block(uid, uid_len)))
            # Try to authenticate with CARD key
            nfc.mfrc630_cmd_load_key(CARDkey)
            for sector in range(0, 16):
                if (nfc.mfrc630_MF_auth(uid, nfc.MFRC630_MF_AUTH_KEY_A, sector * 4)):
                    pycom.rgbled(RGB_GREEN)
                    # Authentication was sucessful, read card data
                    readbuf = bytearray(16)
                    for b in range(0, 4):
                        f_sect = sector * 4 + b
                        len = nfc.mfrc630_MF_read_block(f_sect, readbuf)
                        print("\tSector %s: Block: %s: %s" % (nfc.format_block([sector], 1), nfc.format_block([b], 1), nfc.format_block(readbuf, len)))
                    print("\t"+"_"*32)
                else:
                    print("Authentication denied for sector %s!" % nfc.format_block([sector], 1))
                    pycom.rgbled(RGB_RED)
            # It is necessary to call mfrc630_MF_deauth after authentication
            # Although this is also handled by the reset / init cycle
            nfc.mfrc630_MF_deauth()
    else:
        pycom.rgbled(RGB_BLUE)
    # We could go into power saving mode here... to be investigated
    nfc.mfrc630_cmd_reset()
    time.sleep(.5)
    # Re-Initialise the MFRC630 with settings as these got wiped during reset
    nfc.mfrc630_cmd_init()